@@ -17,14 +17,17 @@ from itertools import takewhile, chain
 from typing import (
     Any,
     Callable,
+    ClassVar,
     Dict,
     Optional,
     List,
     Tuple,
     Iterator,
+    Set,
     TYPE_CHECKING,
 )
 import logging
+from uuid import UUID, uuid4
 
 from tqdm import tqdm
 
@@ -49,7 +52,8 @@ from sqlfluff.core.parser.context import ParseContext
 from sqlfluff.core.templaters.base import TemplatedFile
 
 if TYPE_CHECKING:
-    from sqlfluff.core.rules.base import LintFix  # pragma: no cover
+    from sqlfluff.core.rules import LintFix  # pragma: no cover
+    from sqlfluff.core.parser.segments import RawSegment  # pragma: no cover
 
 # Instantiate the linter logger (only for use in methods involved with fixing.)
 linter_logger = logging.getLogger("sqlfluff.linter")
@@ -196,7 +200,35 @@ class AnchorEditInfo:
         return False  # pragma: no cover
 
 
-class BaseSegment:
+class SegmentMetaclass(type):
+    """The metaclass for segments.
+
+    This metaclass provides pre-computed class attributes
+    based on the defined attributes of specific classes.
+    """
+
+    def __new__(mcs, name, bases, class_dict):
+        """Generate a new class.
+
+        We use the `type` class attribute for the class
+        and it's parent base classes to build up a `set`
+        of types on construction to use in type checking
+        later in the process. Doing it on construction
+        here saves calculating it at runtime for each
+        instance of the class.
+        """
+        class_obj = super().__new__(mcs, name, bases, class_dict)
+        added_type = class_dict.get("type", None)
+        class_types = {added_type} if added_type else set()
+        for base in bases:
+            class_types.update(base._class_types)
+        # NB: We're setting the private value so that some dependent
+        # classes can make their own public property.
+        class_obj._class_types = class_types
+        return class_obj
+
+
+class BaseSegment(metaclass=SegmentMetaclass):
     """The base segment element.
 
     This defines the base element which drives both Lexing, Parsing and Linting.
@@ -215,13 +247,13 @@ class BaseSegment:
     """
 
     # `type` should be the *category* of this kind of segment
-    type = "base"
+    type: ClassVar[str] = "base"
+    _class_types: ClassVar[Set[str]]  # NOTE: Set by SegmentMetaclass
     parse_grammar: Optional[Matchable] = None
     # We define the type here but no value. Subclasses must provide a value.
     match_grammar: Matchable
     comment_separate = False
     optional = False  # NB: See the sequence grammar for details
-    _name: Optional[str] = None
     is_meta = False
     # Are we able to have non-code at the start or end?
     can_start_end_non_code = False
@@ -236,12 +268,10 @@ class BaseSegment:
         self,
         segments,
         pos_marker: Optional[PositionMarker] = None,
-        name: Optional[str] = None,
+        uuid: Optional[UUID] = None,
     ):
         # A cache variable for expandable
         self._is_expandable: Optional[bool] = None
-        # Surrogate name option.
-        self._surrogate_name = name
 
         if len(segments) == 0:  # pragma: no cover
             raise RuntimeError(
@@ -272,6 +302,8 @@ class BaseSegment:
                 )
 
         self.pos_marker = pos_marker
+        # Tracker for matching when things start moving.
+        self.uuid = uuid or uuid4()
 
         self._recalculate_caches()
 
@@ -330,22 +362,6 @@ class BaseSegment:
 
     # ################ PUBLIC PROPERTIES
 
-    @property
-    def name(self) -> str:
-        """The name of this segment.
-
-        The reason for three routes for names is that some subclasses
-        might want to override the name rather than just getting
-        the class name. Instances may also override this with the
-        _surrogate_name.
-
-        Name should be specific to this kind of segment, while `type`
-        should be a higher level descriptor of the kind of segment.
-        For example, the name of `+` is 'plus' but the type might be
-        'binary_operator'.
-        """
-        return self._surrogate_name or self._name or self.__class__.__name__
-
     @property
     def is_expandable(self) -> bool:
         """Return true if it is meaningful to call `expand` on this segment.
@@ -387,6 +403,43 @@ class BaseSegment:
         """Make a string from the segments of this segment."""
         return "".join(seg.raw for seg in self.segments)
 
+    @property
+    def class_types(self) -> Set[str]:
+        """The set of types for this segment."""
+        # NOTE: This version is simple, but some dependent classes
+        # (notably RawSegment) override this with something more
+        # custom.
+        return self._class_types
+
+    @property
+    def expected_form(self) -> str:
+        """What to return to the user when unparsable."""
+        return self.get_type()
+
+    @cached_property
+    def descendant_type_set(self) -> Set[str]:
+        """The set of all contained types.
+
+        This is used for rule crawling.
+
+        NOTE: Does not include the types of the parent segment itself.
+        """
+        return set(
+            chain.from_iterable(
+                seg.descendant_type_set | seg.class_types for seg in self.segments
+            )
+        )
+
+    @cached_property
+    def direct_descendant_type_set(self) -> Set[str]:
+        """The set of all directly child types.
+
+        This is used for rule crawling.
+
+        NOTE: Does not include the types of the parent segment itself.
+        """
+        return set(chain.from_iterable(seg.class_types for seg in self.segments))
+
     @cached_property
     def raw_upper(self) -> str:
         """Make an uppercase string from the segments of this segment."""
@@ -398,7 +451,7 @@ class BaseSegment:
         return sum(seg.matched_length for seg in self.segments)
 
     @cached_property
-    def raw_segments(self) -> List["BaseSegment"]:
+    def raw_segments(self) -> List["RawSegment"]:
         """Returns a list of raw segments in this segment."""
         return self.get_raw_segments()
 
@@ -602,7 +655,7 @@ class BaseSegment:
     # ################ CLASS METHODS
 
     @classmethod
-    def simple(cls, parse_context: ParseContext) -> Optional[List[str]]:
+    def simple(cls, parse_context: ParseContext, crumbs=None) -> Optional[List[str]]:
         """Does this matcher support an uppercase hash matching route?
 
         This should be true if the MATCH grammar is simple. Most more
@@ -610,7 +663,7 @@ class BaseSegment:
         if they wish to be considered simple.
         """
         if cls.match_grammar:
-            return cls.match_grammar.simple(parse_context=parse_context)
+            return cls.match_grammar.simple(parse_context=parse_context, crumbs=crumbs)
         else:  # pragma: no cover TODO?
             # Other segments will either override this method, or aren't
             # simple.
@@ -628,17 +681,9 @@ class BaseSegment:
     @classmethod
     def class_is_type(cls, *seg_type):
         """Is this segment class (or its parent) of the given type."""
-        # Do we match on the type of _this_ class.
-        if cls.type in seg_type:
+        # Use set intersection
+        if cls._class_types.intersection(seg_type):
             return True
-        # If not, check types of parents.
-        for base_class in cls.__bases__:
-            if base_class is object:
-                break
-            elif base_class.type in seg_type:
-                return True
-            elif base_class.type == "base":
-                break
         return False
 
     @classmethod
@@ -722,9 +767,18 @@ class BaseSegment:
                 )
             # Once unified we can deal with it just as a MatchResult
             if m.has_match():
-                return MatchResult(
-                    (cls(segments=m.matched_segments),), m.unmatched_segments
-                )
+                try:
+                    return MatchResult(
+                        (cls(segments=m.matched_segments),), m.unmatched_segments
+                    )
+                except TypeError as err:  # pragma: no cover
+                    # This is an error to assist with debugging dialect design.
+                    # It's most likely that the match_grammar has been set on
+                    # a raw segment which shouldn't happen.
+                    raise TypeError(
+                        f"Error in instantiating {cls.__module__}.{cls.__name__}. Have "
+                        f"you defined a match_grammar on a RawSegment? : {str(err)}"
+                    )
             else:
                 return MatchResult.from_unmatched(segments)
         else:  # pragma: no cover
@@ -746,6 +800,9 @@ class BaseSegment:
             "raw_segments",
             "first_non_whitespace_segment_raw_upper",
             "source_fixes",
+            "full_type_set",
+            "descendant_type_set ",
+            "direct_descendant_type_set ",
         ]:
             self.__dict__.pop(key, None)
 
@@ -774,14 +831,6 @@ class BaseSegment:
         """Is this segment (or its parent) of the given type."""
         return self.class_is_type(*seg_type)
 
-    def get_name(self):
-        """Returns the name of this segment as a string."""
-        return self.name
-
-    def is_name(self, *seg_name):
-        """Is this segment of the given name."""
-        return any(s == self.name for s in seg_name)
-
     def invalidate_caches(self):
         """Invalidate the cached properties.
 
@@ -1143,7 +1192,7 @@ class BaseSegment:
                     + (
                         UnparsableSegment(
                             segments=segments,
-                            expected=self.name,
+                            expected=self.expected_form,
                         ),  # NB: tuple
                     )
                     + post_nc
@@ -1211,9 +1260,9 @@ class BaseSegment:
                 else:
                     seg = todo_buffer.pop(0)
 
-                    # Look for identity not just equality.
+                    # Look for uuid match.
                     # This handles potential positioning ambiguity.
-                    anchor_info: Optional[AnchorEditInfo] = fixes.pop(id(seg), None)
+                    anchor_info: Optional[AnchorEditInfo] = fixes.pop(seg.uuid, None)
                     if anchor_info is not None:
                         seg_fixes = anchor_info.fixes
                         if (
@@ -1225,10 +1274,13 @@ class BaseSegment:
                             seg_fixes.reverse()
 
                         for f in anchor_info.fixes:
-                            assert f.anchor is seg
+                            assert f.anchor.uuid == seg.uuid
                             fixes_applied.append(f)
                             linter_logger.debug(
-                                "Matched fix against segment: %s -> %s", f, seg
+                                "Matched fix for %s against segment: %s -> %s",
+                                rule_code,
+                                f,
+                                seg,
                             )
                             if f.edit_type == "delete":
                                 # We're just getting rid of this segment.
@@ -1271,6 +1323,16 @@ class BaseSegment:
                 # Invalidate any caches
                 self.invalidate_caches()
 
+            # If any fixes applied, do an intermediate reposition. When applying
+            # fixes to children and then trying to reposition them, that recursion
+            # may rely on the parent having already populated positions for any
+            # of the fixes applied there first. This ensures those segments have
+            # working positions to work with.
+            if fixes_applied:
+                seg_buffer = list(
+                    self._position_segments(tuple(seg_buffer), parent_pos=r.pos_marker)
+                )
+
             # Then recurse (i.e. deal with the children) (Requeueing)
             seg_queue = seg_buffer
             seg_buffer = []
@@ -1350,9 +1412,9 @@ class BaseSegment:
         """Group and count fixes by anchor, return dictionary."""
         anchor_info = defaultdict(AnchorEditInfo)  # type: ignore
         for fix in fixes:
-            # :TRICKY: Use segment id() as the dictionary key since
+            # :TRICKY: Use segment uuid as the dictionary key since
             # different segments may compare as equal.
-            anchor_id = id(fix.anchor)
+            anchor_id = fix.anchor.uuid
             anchor_info[anchor_id].add(fix)
         return dict(anchor_info)
 
@@ -1453,11 +1515,18 @@ class BaseSegment:
         else:
             # This segment isn't a literal, but has changed, we need to go deeper.
 
+            # If there's an end of file segment or indent, ignore them just for the
+            # purposes of patch iteration.
+            # NOTE: This doesn't mutate the underlying `self.segments`.
+            segments = self.segments
+            while segments and segments[-1].is_type("end_of_file", "indent"):
+                segments = segments[:-1]
+
             # Iterate through the child segments
             source_idx = self.pos_marker.source_slice.start
             templated_idx = self.pos_marker.templated_slice.start
             insert_buff = ""
-            for seg_idx, segment in enumerate(self.segments):
+            for seg_idx, segment in enumerate(segments):
 
                 # First check for insertions.
                 # We know it's an insertion if it has length but not in the templated
@@ -1563,7 +1632,7 @@ class BracketedSegment(BaseSegment):
         super().__init__(*args, **kwargs)
 
     @classmethod
-    def simple(cls, parse_context: ParseContext) -> Optional[List[str]]:
+    def simple(cls, parse_context: ParseContext, crumbs=None) -> Optional[List[str]]:
         """Simple methods for bracketed and the persitent brackets."""
         start_brackets = [
             start_bracket
@@ -1574,7 +1643,9 @@ class BracketedSegment(BaseSegment):
         ]
         start_simple = []
         for ref in start_brackets:
-            start_simple += parse_context.dialect.ref(ref).simple(parse_context)
+            start_simple += parse_context.dialect.ref(ref).simple(
+                parse_context, crumbs=crumbs
+            )
         return start_simple
 
     @classmethod
@@ -1632,11 +1703,10 @@ class BaseFileSegment(BaseSegment):
         self,
         segments,
         pos_marker=None,
-        name: Optional[str] = None,
         fname: Optional[str] = None,
     ):
         self._file_path = fname
-        super().__init__(segments, pos_marker=pos_marker, name=name)
+        super().__init__(segments, pos_marker=pos_marker)
 
     @property
     def file_path(self):
