@@ -16,7 +16,7 @@ from sqlfluff.core.parser import (
     Delimited,
     KeywordSegment,
     Matchable,
-    NamedParser,
+    TypedParser,
     OneOf,
     Ref,
     RegexLexer,
@@ -42,21 +42,26 @@ mysql_dialect.patch_lexer_matchers(
             "inline_comment",
             r"(-- |#)[^\n]*",
             CommentSegment,
-            segment_kwargs={"trim_start": ("-- ", "#")},
+            segment_kwargs={"trim_start": ("-- ", "#"), "type": "inline_comment"},
         ),
         # Pattern breakdown:
         # (?s)                     DOTALL (dot matches newline)
-        #     ('')+?               group1 match consecutive single quotes
-        #     (?!')                negative lookahead single quote
-        #     |(                   group2 start
-        #         '.*?             single quote wildcard zero or more, lazy
-        #         (?<!'|\\)        negative lookbehind: no single quote or backslash
-        #         (?:'')*          non-capturing group: consecutive single quotes
-        #         '                single quote
+        #     (                    group1 start
+        #         '                single quote (start)
+        #         (?:              non-capturing group: begin
+        #             \\'          MySQL escaped single-quote
+        #             |''          or ANSI escaped single-quotes
+        #             |\\\\        or consecutive [escaped] backslashes
+        #             |[^']        or anything besides a single-quote
+        #         )*               non-capturing group: end (zero or more times)
+        #         '                single quote (end of the single-quoted string)
         #         (?!')            negative lookahead: not single quote
-        #     )                    group2 end
+        #     )                    group1 end
         RegexLexer(
-            "single_quote", r"(?s)('')+?(?!')|('.*?(?<!'|\\)(?:'')*'(?!'))", CodeSegment
+            "single_quote",
+            r"(?s)('(?:\\'|''|\\\\|[^'])*'(?!'))",
+            CodeSegment,
+            segment_kwargs={"type": "single_quote"},
         ),
     ]
 )
@@ -85,11 +90,10 @@ mysql_dialect.sets("reserved_keywords").difference_update(["INDEX"])
 
 
 mysql_dialect.replace(
-    QuotedIdentifierSegment=NamedParser(
+    QuotedIdentifierSegment=TypedParser(
         "back_quote",
-        CodeSegment,
-        name="quoted_identifier",
-        type="identifier",
+        ansi.IdentifierSegment,
+        type="quoted_identifier",
         trim_chars=("`",),
     ),
     LiteralGrammar=ansi_dialect.get_grammar("LiteralGrammar").copy(
@@ -123,6 +127,7 @@ mysql_dialect.replace(
         insert=[
             Ref("SessionVariableNameSegment"),
             Ref("LocalVariableNameSegment"),
+            Ref("VariableAssignmentSegment"),
         ]
     ),
     DateTimeLiteralGrammar=Sequence(
@@ -137,11 +142,10 @@ mysql_dialect.replace(
             optional=True,
         ),
         OneOf(
-            NamedParser(
+            TypedParser(
                 "single_quote",
-                CodeSegment,
-                name="date_constructor_literal",
-                type="literal",
+                ansi.LiteralSegment,
+                type="date_constructor_literal",
             ),
             Ref("NumericLiteralSegment"),
         ),
@@ -151,11 +155,10 @@ mysql_dialect.replace(
             # MySQL allows whitespace-concatenated string literals (#1488).
             # Since these string literals can have comments between them,
             # we use grammar to handle this.
-            NamedParser(
+            TypedParser(
                 "single_quote",
-                CodeSegment,
-                name="quoted_literal",
-                type="literal",
+                ansi.LiteralSegment,
+                type="quoted_literal",
             ),
             min_times=1,
         ),
@@ -168,51 +171,50 @@ mysql_dialect.replace(
     # Odd syntax, but pr
     CharCharacterSetGrammar=Ref.keyword("BINARY"),
     DelimiterGrammar=OneOf(Ref("SemicolonSegment"), Ref("TildeSegment")),
-    TildeSegment=StringParser(
-        "~", SymbolSegment, name="tilde", type="statement_terminator"
-    ),
+    TildeSegment=StringParser("~", SymbolSegment, type="statement_terminator"),
     ParameterNameSegment=RegexParser(
-        r"`?[A-Za-z0-9_]*`?", CodeSegment, name="parameter", type="parameter"
+        r"`?[A-Za-z0-9_]*`?", CodeSegment, type="parameter"
     ),
     SingleIdentifierGrammar=ansi_dialect.get_grammar("SingleIdentifierGrammar").copy(
         insert=[Ref("SessionVariableNameSegment")]
     ),
     AndOperatorGrammar=OneOf(
-        StringParser("AND", KeywordSegment, type="binary_operator"),
-        StringParser(
-            "&&", CodeSegment, name="double_ampersand", type="binary_operator"
-        ),
+        StringParser("AND", ansi.BinaryOperatorSegment),
+        StringParser("&&", ansi.BinaryOperatorSegment),
     ),
     OrOperatorGrammar=OneOf(
-        StringParser("OR", KeywordSegment, type="binary_operator"),
-        StringParser("||", CodeSegment, name="double_pipe", type="binary_operator"),
-        StringParser("XOR", KeywordSegment, type="binary_operator"),
+        StringParser("OR", ansi.BinaryOperatorSegment),
+        StringParser("||", ansi.BinaryOperatorSegment),
+        StringParser("XOR", ansi.BinaryOperatorSegment),
     ),
     NotOperatorGrammar=OneOf(
         StringParser("NOT", KeywordSegment, type="keyword"),
-        StringParser("!", CodeSegment, name="not_operator", type="not_operator"),
+        StringParser("!", CodeSegment, type="not_operator"),
+    ),
+    Expression_C_Grammar=Sequence(
+        Sequence(
+            Ref("SessionVariableNameSegment"),
+            Ref("WalrusOperatorSegment"),
+            optional=True,
+        ),
+        ansi_dialect.get_grammar("Expression_C_Grammar"),
     ),
 )
 
 mysql_dialect.add(
-    DoubleQuotedLiteralSegment=NamedParser(
+    DoubleQuotedLiteralSegment=TypedParser(
         "double_quote",
-        CodeSegment,
-        name="quoted_literal",
-        type="literal",
+        ansi.LiteralSegment,
+        type="quoted_literal",
         trim_chars=('"',),
     ),
-    AtSignLiteralSegment=NamedParser(
-        "at_sign",
-        CodeSegment,
-        name="at_sign_literal",
-        type="literal",
-        trim_chars=("@",),
+    AtSignLiteralSegment=TypedParser(
+        "at_sign_literal",
+        ansi.LiteralSegment,
     ),
     SystemVariableSegment=RegexParser(
         r"@@(session|global)\.[A-Za-z0-9_]+",
         CodeSegment,
-        name="system_variable",
         type="system_variable",
     ),
 )
@@ -292,7 +294,7 @@ class CreateTableStatementSegment(ansi.CreateTableStatementSegment):
                 Sequence(
                     Ref.keyword("DEFAULT", optional=True),
                     Ref("ParameterNameSegment"),
-                    Ref("EqualsSegment"),
+                    Ref("EqualsSegment", optional=True),
                     OneOf(Ref("LiteralGrammar"), Ref("ParameterNameSegment")),
                 ),
             ),
@@ -592,57 +594,128 @@ class DeleteStatementSegment(BaseSegment):
     )
 
 
+class IndexTypeGrammar(BaseSegment):
+    """index_type in `CREATE TABLE` statement.
+
+    https://dev.mysql.com/doc/refman/8.0/en/create-table.html
+    """
+
+    type = "index_type"
+    match_grammar = Sequence(
+        "USING",
+        OneOf("BTREE", "HASH"),
+    )
+
+
+class IndexOptionsSegment(BaseSegment):
+    """index_option in `CREATE TABLE` statement.
+
+    https://dev.mysql.com/doc/refman/8.0/en/create-table.html
+    """
+
+    type = "index_option"
+    match_grammar = AnySetOf(
+        Sequence(
+            "KEY_BLOCK_SIZE",
+            Ref("EqualsSegment", optional=True),
+            Ref("NumericLiteralSegment"),
+        ),
+        Ref("IndexTypeGrammar"),
+        Sequence("WITH", "PARSER", Ref("ObjectReferenceSegment")),
+        Ref("CommentClauseSegment"),
+        OneOf("VISIBLE", "INVISIBLE"),
+        Sequence(
+            "ENGINE_ATTRIBUTE",
+            Ref("EqualsSegment", optional=True),
+            Ref("QuotedLiteralSegment"),
+        ),
+        Sequence(
+            "SECONDARY_ENGINE_ATTRIBUTE",
+            Ref("EqualsSegment", optional=True),
+            Ref("QuotedLiteralSegment"),
+        ),
+    )
+
+
 class TableConstraintSegment(BaseSegment):
-    """A table constraint, e.g. for CREATE TABLE."""
+    """A table constraint, e.g. for CREATE TABLE.
+
+    https://dev.mysql.com/doc/refman/8.0/en/create-table.html
+    """
 
     type = "table_constraint"
     # Later add support for CHECK constraint, others?
     # e.g. CONSTRAINT constraint_1 PRIMARY KEY(column_1)
-    match_grammar = Sequence(
-        Sequence(  # [ CONSTRAINT <Constraint name> ]
-            "CONSTRAINT", Ref("ObjectReferenceSegment"), optional=True
-        ),
-        OneOf(
-            Sequence(  # UNIQUE [INDEX | KEY] [index_name] ( column_name [, ... ] )
-                "UNIQUE",
-                OneOf("INDEX", "KEY", optional=True),
-                Ref("ObjectReferenceSegment", optional=True),
-                Ref("BracketedColumnReferenceListGrammar"),
-                # Later add support for index_parameters?
+    match_grammar = OneOf(
+        Sequence(
+            Sequence(  # [ CONSTRAINT <Constraint name> ]
+                "CONSTRAINT", Ref("ObjectReferenceSegment"), optional=True
             ),
-            Sequence(  # PRIMARY KEY ( column_name [, ... ] ) index_parameters
-                Ref("PrimaryKeyGrammar"),
-                # Columns making up PRIMARY KEY constraint
-                Ref("BracketedColumnReferenceListGrammar"),
-                # Later add support for index_parameters?
-            ),
-            Sequence(  # FOREIGN KEY ( column_name [, ... ] )
-                # REFERENCES reftable [ ( refcolumn [, ... ] ) ]
-                Ref("ForeignKeyGrammar"),
-                # Local columns making up FOREIGN KEY constraint
-                Ref("BracketedColumnReferenceListGrammar"),
-                "REFERENCES",
-                Ref("ColumnReferenceSegment"),
-                # Foreign columns making up FOREIGN KEY constraint
-                Ref("BracketedColumnReferenceListGrammar"),
-                # Later add support for [MATCH FULL/PARTIAL/SIMPLE] ?
-                # Later add support for [ ON DELETE/UPDATE action ] ?
-                AnyNumberOf(
-                    Sequence(
-                        "ON",
-                        OneOf("DELETE", "UPDATE"),
-                        OneOf(
-                            "RESTRICT",
-                            "CASCADE",
-                            Sequence("SET", "NULL"),
-                            Sequence("NO", "ACTION"),
-                            Sequence("SET", "DEFAULT"),
+            OneOf(
+                # UNIQUE [INDEX | KEY] [index_name] [index_type] (key_part,...)
+                # [index_option] ...
+                Sequence(
+                    "UNIQUE",
+                    OneOf("INDEX", "KEY", optional=True),
+                    Ref("IndexReferenceSegment", optional=True),
+                    Ref("IndexTypeGrammar", optional=True),
+                    Ref("BracketedColumnReferenceListGrammar"),
+                    Ref("IndexOptionsSegment", optional=True),
+                ),
+                # PRIMARY KEY [index_type] (key_part,...) [index_option] ...
+                Sequence(
+                    Ref("PrimaryKeyGrammar"),
+                    Ref("IndexTypeGrammar", optional=True),
+                    # Columns making up PRIMARY KEY constraint
+                    Ref("BracketedColumnReferenceListGrammar"),
+                    Ref("IndexOptionsSegment", optional=True),
+                ),
+                # FOREIGN KEY [index_name] (col_name,...) reference_definition
+                Sequence(
+                    # REFERENCES reftable [ ( refcolumn [, ... ] ) ]
+                    Ref("ForeignKeyGrammar"),
+                    # Local columns making up FOREIGN KEY constraint
+                    Ref("BracketedColumnReferenceListGrammar"),
+                    "REFERENCES",
+                    Ref("ColumnReferenceSegment"),
+                    # Foreign columns making up FOREIGN KEY constraint
+                    Ref("BracketedColumnReferenceListGrammar"),
+                    # Later add support for [MATCH FULL/PARTIAL/SIMPLE] ?
+                    # Later add support for [ ON DELETE/UPDATE action ] ?
+                    AnyNumberOf(
+                        Sequence(
+                            "ON",
+                            OneOf("DELETE", "UPDATE"),
+                            OneOf(
+                                "RESTRICT",
+                                "CASCADE",
+                                Sequence("SET", "NULL"),
+                                Sequence("NO", "ACTION"),
+                                Sequence("SET", "DEFAULT"),
+                            ),
+                            optional=True,
                         ),
-                        optional=True,
                     ),
                 ),
             ),
         ),
+        # {INDEX | KEY} [index_name] [index_type] (key_part,...) [index_option] ...
+        Sequence(
+            OneOf("INDEX", "KEY"),
+            Ref("IndexReferenceSegment", optional=True),
+            Ref("IndexTypeGrammar", optional=True),
+            Ref("BracketedColumnReferenceListGrammar"),
+            Ref("IndexOptionsSegment", optional=True),
+        ),
+        # {FULLTEXT | SPATIAL} [INDEX | KEY] [index_name] (key_part,...)
+        # [index_option] ...
+        Sequence(
+            OneOf("FULLTEXT", "SPATIAL"),
+            OneOf("INDEX", "KEY", optional=True),
+            Ref("IndexReferenceSegment", optional=True),
+            Ref("BracketedColumnReferenceListGrammar"),
+            Ref("IndexOptionsSegment", optional=True),
+        ),
     )
 
 
@@ -669,13 +742,11 @@ class IntervalExpressionSegment(BaseSegment):
 
 mysql_dialect.add(
     OutputParameterSegment=StringParser(
-        "OUT", SymbolSegment, name="inputparameter", type="parameter_direction"
-    ),
-    InputParameterSegment=StringParser(
-        "IN", SymbolSegment, name="outputparameter", type="parameter_direction"
+        "OUT", SymbolSegment, type="parameter_direction"
     ),
+    InputParameterSegment=StringParser("IN", SymbolSegment, type="parameter_direction"),
     InputOutputParameterSegment=StringParser(
-        "INOUT", SymbolSegment, name="inputoutputparameter", type="parameter_direction"
+        "INOUT", SymbolSegment, type="parameter_direction"
     ),
     ProcedureParameterGrammar=OneOf(
         Sequence(
@@ -693,15 +764,19 @@ mysql_dialect.add(
     LocalVariableNameSegment=RegexParser(
         r"`?[a-zA-Z0-9_$]*`?",
         CodeSegment,
-        name="declared_variable",
         type="variable",
     ),
     SessionVariableNameSegment=RegexParser(
         r"[@][a-zA-Z0-9_$]*",
         CodeSegment,
-        name="declared_variable",
         type="variable",
     ),
+    WalrusOperatorSegment=StringParser(":=", SymbolSegment, type="assignment_operator"),
+    VariableAssignmentSegment=Sequence(
+        Ref("SessionVariableNameSegment"),
+        Ref("WalrusOperatorSegment"),
+        Ref("BaseExpressionElementGrammar"),
+    ),
     BooleanDynamicSystemVariablesGrammar=OneOf(
         # Boolean dynamic system varaiables can be set to ON/OFF, TRUE/FALSE, or 0/1:
         # https://dev.mysql.com/doc/refman/8.0/en/dynamic-system-variables.html
@@ -719,6 +794,7 @@ mysql_dialect.insert_lexer_matchers(
             "at_sign",
             r"@@?[a-zA-Z0-9_$]*(\.[a-zA-Z0-9_$]+)?",
             CodeSegment,
+            segment_kwargs={"type": "at_sign_literal", "trim_chars": ("@",)},
         ),
     ],
     before="code",
@@ -741,6 +817,14 @@ mysql_dialect.insert_lexer_matchers(
 )
 
 
+mysql_dialect.insert_lexer_matchers(
+    [
+        StringLexer("walrus_operator", ":=", CodeSegment),
+    ],
+    before="equals",
+)
+
+
 class RoleReferenceSegment(ansi.RoleReferenceSegment):
     """A reference to an account, role, or user.
 
@@ -997,15 +1081,15 @@ class AlterTableStatementSegment(BaseSegment):
                     Ref.keyword("UNIQUE", optional=True),
                     OneOf("INDEX", "KEY", optional=True),
                     Ref("IndexReferenceSegment"),
-                    Sequence("USING", OneOf("BTREE", "HASH"), optional=True),
+                    Ref("IndexTypeGrammar", optional=True),
                     Ref("BracketedColumnReferenceListGrammar"),
                     AnySetOf(
                         Sequence(
                             "KEY_BLOCK_SIZE",
-                            Ref("EqualsSegment"),
+                            Ref("EqualsSegment", optional=True),
                             Ref("NumericLiteralSegment"),
                         ),
-                        Sequence("USING", OneOf("BTREE", "HASH")),
+                        Ref("IndexTypeGrammar"),
                         Sequence("WITH", "PARSER", Ref("ObjectReferenceSegment")),
                         Ref("CommentClauseSegment"),
                         OneOf("VISIBLE", "INVISIBLE"),
@@ -1092,7 +1176,10 @@ class SetAssignmentStatementSegment(BaseSegment):
                 OneOf(
                     Ref("SessionVariableNameSegment"), Ref("LocalVariableNameSegment")
                 ),
-                Ref("EqualsSegment"),
+                OneOf(
+                    Ref("EqualsSegment"),
+                    Ref("WalrusOperatorSegment"),
+                ),
                 AnyNumberOf(
                     Ref("QuotedLiteralSegment"),
                     Ref("DoubleQuotedLiteralSegment"),
