@@ -45,7 +45,9 @@ redshift_dialect.sets("reserved_keywords").update(
 )
 
 redshift_dialect.sets("bare_functions").clear()
-redshift_dialect.sets("bare_functions").update(["current_date", "sysdate"])
+redshift_dialect.sets("bare_functions").update(
+    ["current_date", "sysdate", "current_timestamp"]
+)
 
 redshift_dialect.sets("date_part_function_name").update(
     ["DATEADD", "DATEDIFF", "EXTRACT", "DATE_PART"]
@@ -170,9 +172,8 @@ redshift_dialect.replace(
             # must only contain digits, letters, underscore, and $ but
             # can’t be all digits.
             r"#?([A-Z_]+|[0-9]+[A-Z_$])[A-Z0-9_$]*",
-            CodeSegment,
-            name="naked_identifier",
-            type="identifier",
+            ansi.IdentifierSegment,
+            type="naked_identifier",
             anti_template=r"^(" + r"|".join(dialect.sets("reserved_keywords")) + r")$",
         )
     ),
@@ -2248,8 +2249,11 @@ class TableExpressionSegment(ansi.TableExpressionSegment):
     """
 
     match_grammar = ansi.TableExpressionSegment.match_grammar.copy(
-        insert=[Ref("ObjectUnpivotSegment", optional=True)],
-        before=Ref("TableReferenceSegment", optional=True),
+        insert=[
+            Ref("ObjectUnpivotSegment", optional=True),
+            Ref("ArrayUnnestSegment", optional=True),
+        ],
+        before=Ref("TableReferenceSegment"),
     )
 
 
@@ -2270,6 +2274,41 @@ class ObjectUnpivotSegment(BaseSegment):
     )
 
 
+class ArrayAccessorSegment(ansi.ArrayAccessorSegment):
+    """Array element accessor.
+
+    Redshift allows multiple levels of array access, like Postgres,
+    but it
+    * doesn't allow ranges like `myarray[1:2]`
+    * does allow function or column expressions `myarray[idx]`
+    """
+
+    match_grammar = Sequence(
+        AnyNumberOf(
+            Bracketed(
+                OneOf(Ref("NumericLiteralSegment"), Ref("ExpressionSegment")),
+                bracket_type="square",
+            )
+        )
+    )
+
+
+class ArrayUnnestSegment(BaseSegment):
+    """Array unnesting.
+
+    https://docs.aws.amazon.com/redshift/latest/dg/query-super.html
+    """
+
+    type = "array_unnesting"
+    match_grammar: Matchable = Sequence(
+        Ref("ObjectReferenceSegment"),
+        "AS",
+        Ref("SingleIdentifierGrammar"),
+        "AT",
+        Ref("SingleIdentifierGrammar"),
+    )
+
+
 class CallStatementSegment(BaseSegment):
     """A `CALL` statement.
 
@@ -2289,3 +2328,76 @@ class SelectClauseModifierSegment(postgres.SelectClauseModifierSegment):
     match_grammar = postgres.SelectClauseModifierSegment.match_grammar.copy(
         insert=[Sequence("TOP", Ref("NumericLiteralSegment"))],
     )
+
+
+class ConvertFunctionNameSegment(BaseSegment):
+    """CONVERT function name segment.
+
+    Function taking a data type identifier and an expression.
+    An alternative to CAST.
+    """
+
+    type = "function_name"
+    match_grammar = Sequence("CONVERT")
+
+
+class FunctionSegment(ansi.FunctionSegment):
+    """A scalar or aggregate function.
+
+    Maybe in the future we should distinguish between
+    aggregate functions and other functions. For now
+    we treat them the same because they look the same
+    for our purposes.
+    """
+
+    type = "function"
+    match_grammar: Matchable = OneOf(
+        Sequence(
+            # Treat functions which take date parts separately
+            # So those functions parse date parts as DatetimeUnitSegment
+            # rather than identifiers.
+            Sequence(
+                Ref("DatePartFunctionNameSegment"),
+                Bracketed(
+                    Delimited(
+                        Ref("DatetimeUnitSegment"),
+                        Ref(
+                            "FunctionContentsGrammar",
+                            # The brackets might be empty for some functions...
+                            optional=True,
+                            ephemeral_name="FunctionContentsGrammar",
+                        ),
+                    )
+                ),
+            ),
+        ),
+        Sequence(
+            Sequence(
+                Ref(
+                    "FunctionNameSegment",
+                    exclude=OneOf(
+                        Ref("DatePartFunctionNameSegment"),
+                        Ref("ValuesClauseSegment"),
+                        Ref("ConvertFunctionNameSegment"),
+                    ),
+                ),
+                Bracketed(
+                    Ref(
+                        "FunctionContentsGrammar",
+                        # The brackets might be empty for some functions...
+                        optional=True,
+                        ephemeral_name="FunctionContentsGrammar",
+                    )
+                ),
+            ),
+            Ref("PostFunctionGrammar", optional=True),
+        ),
+        Sequence(
+            Ref("ConvertFunctionNameSegment"),
+            Bracketed(
+                Ref("DatatypeSegment"),
+                Ref("CommaSegment"),
+                Ref("ExpressionSegment"),
+            ),
+        ),
+    )
