@@ -22,7 +22,6 @@ CURRENT_DATE
 CURRENT_TIME
 CURRENT_TIMESTAMP
 CURRENT_USER
-DATABASE
 DELETE
 DISTINCT
 DROP
@@ -71,7 +70,6 @@ RLIKE
 ROW
 ROWS
 SAMPLE
-SCHEMA
 SELECT
 SET
 SOME
@@ -112,6 +110,7 @@ APPLY
 ARRAY
 ASC
 AT
+ATTACH
 AUTHORIZATION
 AUTHORIZATIONS
 AUTO
@@ -126,10 +125,16 @@ AVRO
 AWS_KEY_ID
 AWS_ROLE
 AWS_SECRET_KEY
+AWS_SNS
+AWS_SNS_ROLE_ARN
 AWS_SNS_TOPIC
+AWS_SNS_TOPIC_ARN
 AWS_TOKEN
 AZURE
+AZURE_EVENT_GRID
+AZURE_EVENT_GRID_TOPIC_ENDPOINT
 AZURE_SAS_TOKEN
+AZURE_STORAGE_QUEUE_PRIMARY_URI
 AZURE_TENANT_ID
 BASE64
 BEFORE
@@ -188,6 +193,7 @@ CURRENT_WAREHOUSE
 CYCLE
 DATA
 DATA_RETENTION_TIME_IN_DAYS
+DATABASE
 DATABASES
 DATE
 DATE_FORMAT
@@ -200,6 +206,7 @@ DEFLATE
 DELEGATED
 DESC
 DESCRIBE
+DIRECTION
 DIRECTORY
 DISABLE
 DISABLE_AUTO_CONVERT
@@ -218,9 +225,12 @@ END
 ENFORCE_LENGTH
 ENFORCED
 ENUM
+ERROR_INTEGRATION
 ERROR_ON_COLUMN_COUNT_MISMATCH
 ESCAPE
+ESCAPE_UNENCLOSED_FIELD
 EXCEPT
+EXCHANGE
 EXECUTE
 EXECUTION
 EXPLAIN
@@ -245,6 +255,9 @@ FORMATS
 FUNCTION
 FUNCTIONS
 FUTURE
+GCP_PUBSUB
+GCP_PUBSUB_SUBSCRIPTION_NAME
+GCP_PUBSUB_TOPIC_NAME
 GCS
 GET
 GLOBAL
@@ -262,6 +275,7 @@ IGNORE
 IGNORE_UTF8_ERRORS
 IMMEDIATE
 IMMUTABLE
+IMPORT
 IMPORTED
 INDEX
 INITIALLY
@@ -282,6 +296,7 @@ LAST_QUERY_ID
 LAST_TRANSACTION
 LIMIT
 LIST
+LISTING
 LOCAL
 LOCATION
 LOCKS
@@ -309,6 +324,7 @@ MIN_CLUSTER_COUNT
 MINVALUE
 ML
 MODEL
+MODIFIED_AFTER
 MODIFY
 MONITOR
 NAME
@@ -323,6 +339,7 @@ NONE
 NOORDER
 NOTIFICATION
 NOTIFICATION_INTEGRATION
+NOTIFICATION_PROVIDER
 NULL_IF
 NULLS
 OBJECT
@@ -338,9 +355,11 @@ OPTION
 OPTIONS
 ORC
 ORGANIZATION
+OUTBOUND
 OUTER
 OVER
 OVERLAPS
+OVERRIDE
 OVERWRITE
 OWNER
 OWNERSHIP
@@ -353,12 +372,14 @@ PATTERN
 PER
 PERMUTE
 PIPE
+PIPE_EXECUTION_PAUSED
 PIPES
 PIVOT
 POLICIES
 POLICY
 PRECEDING
 PRECISION
+PREFIX
 PRESERVE_SPACE
 PRIMARY
 PRIOR
@@ -369,6 +390,7 @@ PUBLIC
 PURGE
 PUT
 QUERIES
+QUEUE
 RANGE
 RAW_DEFLATE
 READ
@@ -392,6 +414,7 @@ RESOURCE_MONITOR
 RESPECT
 RESPONSE_TRANSLATOR
 RESTRICT
+RESTRICTIONS
 RESULT
 RESUME
 RETURN_ALL_ERRORS
@@ -405,11 +428,13 @@ ROLLBACK
 ROLLUP
 ROUTINE
 ROUTINES
+ROW
 REQUEST_TRANSLATOR
 RUNNING
 S3
 SCALING_POLICY
 SCHEDULE
+SCHEMA
 SCHEMAS
 SEARCH
 SECONDARY
@@ -517,6 +542,7 @@ VOLATILE
 WAIT_FOR_COMPLETION
 WAREHOUSE
 WAREHOUSE_SIZE
+WAREHOUSE_TYPE
 WAREHOUSES
 WINDOW
 WITH
