@@ -9,8 +9,9 @@ from sqlfluff.core.parser import (
     WhitespaceSegment,
 )
 
-from sqlfluff.core.rules.base import LintFix, LintResult, RuleContext
-from sqlfluff.core.rules.functional import sp
+from sqlfluff.core.rules import LintFix, LintResult, RuleContext
+from sqlfluff.utils.functional import sp, Segments
+from sqlfluff.core.rules.crawlers import SegmentSeekerCrawler
 from sqlfluff.core.rules.doc_decorators import (
     document_configuration,
     document_fix_compatible,
@@ -26,7 +27,7 @@ class Rule_L016(Rule_L003):
     """Line is too long."""
 
     groups = ("all", "core")
-    needs_raw_stack = True
+    crawl_behaviour = SegmentSeekerCrawler({"newline"}, provide_raw_stack=True)
     _adjust_anchors = True
     _check_docstring = False
 
@@ -255,7 +256,7 @@ class Rule_L016(Rule_L003):
                         is_pause = False
                     else:
                         # We're not in a pause (or not in a pause yet)
-                        if seg.name == "comma":  # or seg.is_type('binary_operator')
+                        if seg.is_type("comma"):  # or seg.is_type('binary_operator')
                             if segment_buff:
                                 # End the previous section, start a comma/operator.
                                 # Any whitespace is added to the segment
@@ -386,7 +387,7 @@ class Rule_L016(Rule_L003):
         while True:
             if len(raw_stack) >= abs(idx):
                 s = raw_stack[idx]
-                if s.name == "newline":
+                if s.is_type("newline"):
                     break
                 else:
                     working_buff.insert(0, s)
@@ -427,12 +428,15 @@ class Rule_L016(Rule_L003):
             return len(segment.raw)
 
     def _compute_source_length(
-        self, segments: Sequence[BaseSegment], memory: dict
+        self,
+        segments: Sequence[BaseSegment],
+        literals_in_comments: Sequence[BaseSegment],
     ) -> int:
         line_len = 0
         seen_slices = set()
         for segment in segments:
-            if self.ignore_comment_clauses and segment in memory["comment_clauses"]:
+            if segment in literals_in_comments:
+                self.logger.debug("Not counting literal in comment: %s", segment)
                 continue
 
             assert segment.pos_marker
@@ -481,28 +485,50 @@ class Rule_L016(Rule_L003):
         self.ignore_comment_lines: bool
         self.ignore_comment_clauses: bool
 
-        if not context.memory:
-            memory: dict = {"comment_clauses": set()}
-        else:
-            memory = context.memory
-        if context.segment.name == "newline":
-            # iterate to buffer the whole line up to this point
-            this_line = self._gen_line_so_far(context.raw_stack)
-        else:
-            if self.ignore_comment_clauses and context.segment.is_type(
-                "comment_clause", "comment_equals_clause"
-            ):
-                comment_segment = context.functional.segment.children().first(
-                    sp.is_name("quoted_literal")
-                )
-                if comment_segment:
-                    memory["comment_clauses"].add(comment_segment.get())
+        assert context.segment.is_type("newline")
 
-            # Otherwise we're all good
-            return LintResult(memory=memory)
+        # iterate to buffer the whole line up to this point
+        this_line = self._gen_line_so_far(context.raw_stack)
+
+        # Do any literals on this line belong to a comment?
+        literals_in_comments: Sequence[BaseSegment] = []
+        if self.ignore_comment_clauses:
+            quoted_literals = Segments(*this_line).select(
+                select_if=sp.is_type("quoted_literal")
+            )
+            if quoted_literals:
+                self.logger.debug(
+                    "Comment Checking: Found quoted literals: %s", quoted_literals
+                )
+                root = context.parent_stack[0]
+
+                def is_in_comment(seg):
+                    """This evaluates whether the parent segment is a comment.
+
+                    We use path_to to get the stack of segments from the root
+                    to the given segment. The last element of that stack is the
+                    given segment (`seg` in this function), the second last
+                    is the parent of that segment: i.e. path[-2].
+                    """
+                    path = root.path_to(seg)
+                    # It's unlikely that the path will be less than 2 segments
+                    # long. That would imply that we've passed the root segment
+                    # itself - but in that case - we should conclude it's not
+                    # in a comment.
+                    if len(path) < 2:
+                        return False  # pragma: no cover
+                    parent = path[-2]
+                    return parent.is_type("comment_clause", "comment_equals_clause")
+
+                literals_in_comments = quoted_literals.select(select_if=is_in_comment)
+                if literals_in_comments:
+                    self.logger.debug(
+                        "Comment Checking: Literals in comments: %s",
+                        literals_in_comments,
+                    )
 
         # Now we can work out the line length and deal with the content
-        line_len = self._compute_source_length(this_line, memory)
+        line_len = self._compute_source_length(this_line, literals_in_comments)
         if line_len > self.max_line_length:
             # Problem, we'll be reporting a violation. The
             # question is, can we fix it?
@@ -510,7 +536,7 @@ class Rule_L016(Rule_L003):
             # We'll need the indent, so let's get it for fixing.
             line_indent = []
             for s in this_line:
-                if s.name == "whitespace":
+                if s.is_type("whitespace"):
                     line_indent.append(s)
                 else:
                     break
@@ -520,23 +546,30 @@ class Rule_L016(Rule_L003):
             # these long lines will likely be single line Jinja comments.
             # They will remain as unfixable.
             if this_line[-1].type == "placeholder":
-                self.logger.info("Unfixable template segment: %s", this_line[-1])
-                return LintResult(anchor=context.segment, memory=memory)
+                if (
+                    this_line[-1].block_type != "comment"  # type: ignore[attr-defined]
+                    or not self.ignore_comment_clauses
+                ):
+                    self.logger.info("Unfixable template segment: %s", this_line[-1])
+                    return LintResult(anchor=context.segment)
+                else:
+                    return LintResult()
 
             # Does the line end in an inline comment that we can move back?
-            if this_line[-1].name == "inline_comment":
+            if this_line[-1].is_type("inline_comment"):
                 # Is this line JUST COMMENT (with optional preceding whitespace) if
                 # so, user will have to fix themselves.
                 if len(this_line) == 1 or all(
-                    elem.name == "whitespace" or elem.is_meta for elem in this_line[:-1]
+                    elem.is_type("whitespace") or elem.is_meta
+                    for elem in this_line[:-1]
                 ):
                     self.logger.info(
                         "Unfixable inline comment, alone on line: %s", this_line[-1]
                     )
                     if self.ignore_comment_lines:
-                        return LintResult(memory=memory)
+                        return LintResult()
                     else:
-                        return LintResult(anchor=context.segment, memory=memory)
+                        return LintResult(anchor=context.segment)
 
                 self.logger.info(
                     "Attempting move of inline comment at end of line: %s",
@@ -546,9 +579,8 @@ class Rule_L016(Rule_L003):
                 delete_buffer = [LintFix.delete(this_line[-1])]
                 idx = -2
                 while True:
-                    if (
-                        len(this_line) >= abs(idx)
-                        and this_line[idx].name == "whitespace"
+                    if len(this_line) >= abs(idx) and this_line[idx].is_type(
+                        "whitespace"
                     ):
                         delete_buffer.append(LintFix.delete(this_line[idx]))
                         idx -= 1
@@ -559,7 +591,7 @@ class Rule_L016(Rule_L003):
                     cast(RawSegment, context.segment),
                 ]
                 if (
-                    self._compute_source_length(create_elements, memory)
+                    self._compute_source_length(create_elements, literals_in_comments)
                     > self.max_line_length
                 ):
                     # The inline comment is NOT on a line by itself, but even if
@@ -573,9 +605,9 @@ class Rule_L016(Rule_L003):
                         this_line[-1],
                     )
                     if self.ignore_comment_lines:
-                        return LintResult(memory=memory)
+                        return LintResult()
                     else:
-                        return LintResult(anchor=context.segment, memory=memory)
+                        return LintResult(anchor=context.segment)
                 # Create a newline before this one with the existing comment, an
                 # identical indent AND a terminating newline, copied from the current
                 # target segment.
@@ -583,11 +615,10 @@ class Rule_L016(Rule_L003):
                 return LintResult(
                     anchor=context.segment,
                     fixes=delete_buffer + create_buffer,
-                    memory=memory,
                 )
 
             fixes = self._eval_line_for_breaks(this_line)
             if fixes:
-                return LintResult(anchor=context.segment, fixes=fixes, memory=memory)
-            return LintResult(anchor=context.segment, memory=memory)
-        return LintResult(memory=memory)
+                return LintResult(anchor=context.segment, fixes=fixes)
+            return LintResult(anchor=context.segment)
+        return LintResult()
