@@ -4,21 +4,30 @@ from typing import cast, List, Optional, Tuple
 
 from sqlfluff.core.dialects.base import Dialect
 from sqlfluff.core.dialects.common import AliasInfo
-from sqlfluff.core.rules.analysis.select_crawler import (
+from sqlfluff.utils.analysis.select_crawler import (
     Query as SelectCrawlerQuery,
     SelectCrawler,
 )
-from sqlfluff.core.rules.base import (
+from sqlfluff.core.rules import (
     BaseRule,
     LintResult,
     RuleContext,
     EvalResultType,
 )
+from sqlfluff.core.rules.crawlers import SegmentSeekerCrawler
 from sqlfluff.core.rules.doc_decorators import document_configuration, document_groups
-from sqlfluff.core.rules.functional import sp
+from sqlfluff.utils.functional import sp, FunctionalContext
 from sqlfluff.core.rules.reference import object_ref_matches_table
 
 
+_START_TYPES = [
+    "delete_statement",
+    "merge_statement",
+    "select_statement",
+    "update_statement",
+]
+
+
 @dataclass
 class L026Query(SelectCrawlerQuery):
     """SelectCrawler Query with custom L026 info."""
@@ -61,6 +70,7 @@ class Rule_L026(BaseRule):
 
     groups = ("all", "core")
     config_keywords = ["force_enable"]
+    crawl_behaviour = SegmentSeekerCrawler(set(_START_TYPES))
     _dialects_disabled_by_default = ["bigquery", "hive", "redshift", "soql", "sparksql"]
 
     def _eval(self, context: RuleContext) -> EvalResultType:
@@ -74,20 +84,19 @@ class Rule_L026(BaseRule):
             return LintResult()
 
         violations: List[LintResult] = []
-        start_types = ["select_statement", "delete_statement", "update_statement"]
-        if context.segment.is_type(
-            *start_types
-        ) and not context.functional.parent_stack.any(sp.is_type(*start_types)):
+        if not FunctionalContext(context).parent_stack.any(sp.is_type(*_START_TYPES)):
             dml_target_table: Optional[Tuple[str, ...]] = None
+            self.logger.debug("Trigger on: %s", context.segment)
             if not context.segment.is_type("select_statement"):
                 # Extract first table reference. This will be the target
-                # table in a DELETE or UPDATE statement.
+                # table in a DML statement.
                 table_reference = next(
                     context.segment.recursive_crawl("table_reference"), None
                 )
                 if table_reference:
                     dml_target_table = self._table_ref_as_tuple(table_reference)
 
+            self.logger.debug("DML Reference Table: %s", dml_target_table)
             # Verify table references in any SELECT statements found in or
             # below context.segment in the parser tree.
             crawler = SelectCrawler(
@@ -123,10 +132,19 @@ class Rule_L026(BaseRule):
         # For each query...
         for selectable in query.selectables:
             select_info = selectable.select_info
+            self.logger.debug(
+                "Selectable: %s",
+                selectable,
+            )
             if select_info:
                 # Record the available tables.
                 query.aliases += select_info.table_aliases
                 query.standalone_aliases += select_info.standalone_aliases
+                self.logger.debug(
+                    "Aliases: %s %s",
+                    [alias.ref_str for alias in select_info.table_aliases],
+                    select_info.standalone_aliases,
+                )
 
                 # Try and resolve each reference to a value in query.aliases (or
                 # in an ancestor query).
@@ -205,7 +223,7 @@ class Rule_L026(BaseRule):
                     r, tbl_refs, dml_target_table, cast(L026Query, query.parent)
                 )
             # No parent query. If there's a DML statement at the root, check its
-            # target table.
+            # target table or alias.
             elif not dml_target_table or not object_ref_matches_table(
                 possible_references, [dml_target_table]
             ):
